const net = require("net");
const {exec} = require("child_process");
const async = require("async");
const parseXML = require("xml2js").parseString;

const wmic = function(cls, keys, callback) {
  const command = `wmic path ${cls} get ${keys.join(",")} /format:rawxml`;
  return exec(command, function(error, stdout) {
    if (error !== null) { return callback(error); }
    return parseXML(stdout, {
      trim: true,
      normalize: true,
      normalizeTags: true,
      mergeAttrs: true,
      async: true
    }, function(error, result) {
      const records = [];
      for (const r of Array.from(result.command.results[0].cim[0].instance)) {
        const record = {};
        for (const p of Array.from(r.property || [])) {
          record[p.NAME] = (p.value && p.value[0]) || "";
        }
        for (const p of Array.from(r["property.array"] || [])) {
          const val = p["value.array"];
          record[p.NAME] = (val && p["value.array"][0].value) || "";
        }
        records.push(record);
      }
      return callback(error, records);
    });
  });
};

const getAdapterConfig = callback => {
  wmic("Win32_NetworkAdapterConfiguration",
    ["Index", "IPEnabled", "DefaultIPGateway"],
    (error, records) => callback(error, records));
};

const getAdapter = callback => {
  wmic("Win32_NetworkAdapter",
    ["Index", "NetConnectionID"],
    (error, records) => callback(error, records));
};

const getDefaultGateway = callback => {
  getAdapterConfig(function(error, records) {
    if (error !== null) { return callback(error); }
    const data = {};
    for (const record of Array.from(records)) {
      if ((record["IPEnabled"] === null)) return;
      if ((record["DefaultIPGateway"] === null)) return;
      if ((record["Index"] === null)) return;
      if (record["IPEnabled"] !== "TRUE") return;
      if (record["DefaultIPGateway"].length === 0) return;
      if (isNaN(parseInt(record["Index"]))) return;
      const index = record["Index"];
      const defaultGateway = record["DefaultIPGateway"];
      for (const address of Array.from(defaultGateway)) {
        switch (net.isIP(address)) {
        case 4:
          data[index] = data[index] || [];
          data[index].push({family: "IPv4", address});
          break;
        case 6:
          data[index] = data[index] || [];
          data[index].push({family: "IPv6", address});
          break;
        default:
          return callback(new Error(`${address} is not IP address`));
        }
      }
    }
    return callback(null, data);
  });
};

const getAdapterNameByIndex = (index, callback) => {
  getAdapter(function(error, records) {
    if (error !== null) { return callback(error); }
    for (const record of Array.from(records)) {
      if (record["Index"] === index) {
        return callback(null, record["NetConnectionID"]);
      }
    }
    return callback(new Error(`inteface ${index} is not available`));
  });
};

const getDefaultNetwork = callback => {
  getDefaultGateway(function(error, gateways) {
    if (error !== null) { return callback(error); }
    const indexes = Object.keys(gateways);
    return async.map(indexes,
      (index, callback) =>
        getAdapterNameByIndex(index, function(error, name) {
          if (error !== null) { return callback(error); }
          const iface = {index, name};
          return callback(null, iface);
        })
      ,
      function(error, ifaces) {
        if (error !== null) { return callback(error); }
        const data = {};
        for (const iface of Array.from(ifaces)) {
          data[iface.name] = gateways[iface.index];
        }
        return callback(null, data);
      });
  });
};

module.exports = callback => {
  getDefaultNetwork(function(error, data) {
    if (error !== null) { return callback(null, {}); }
    return callback(null, data);
  });
};
